@@ -1,27 +1,150 @@
+import os
+import uuid
+
+import orjson
 import requests
 import time
-import json
+
 import urllib.parse
 import logging
 
-from typing import Dict, Callable, Union
+from typing import Dict, Callable, Union, Optional
 
+from requests import Response
+
+from instauto.api.actions.stub import StubMixin
 from instauto.api.structs import DeviceProfile, IGProfile, State, Method
 from instauto.api.constants import API_BASE_URL
-from instauto.api.exceptions import WrongMethodException, IncorrectLoginDetails, InvalidUserId, BadResponse
+from instauto.api.exceptions import IncorrectLoginDetails, InvalidUserId, BadResponse, AuthorizationError
 
 logger = logging.getLogger(__name__)
 logging.captureWarnings(True)
 
 
-class RequestMixin:
-    ig_profile: IGProfile
-    device_profile: DeviceProfile
-    state: State
-    _user_agent: str
-    _encrypt_password: Callable
-    _session: requests.Session
-    _request_finished_callbacks: list
+class RequestMixin(StubMixin):
+    def _request(self,
+                 endpoint: str,
+                 method: Method,
+                 query: dict = None,
+                 body: Union[dict, list, bytes] = None,
+                 headers: Dict[str, str] = None,
+                 add_default_headers: bool = None,
+                 sign_request: bool = None
+                 ) -> requests.Response:
+        query = query or {}
+        body = body or {}
+        add_default_headers = add_default_headers or True
+        headers = headers or {}
+        sign_request = sign_request or False
+        self._request_preflight_check(method, endpoint, body, sign_request)
+
+        url = self._prepare_url(endpoint, query)
+        headers = self._build_headers(add_default_headers, headers, method)
+        body = self._build_body(body, sign_request)
+
+        try:
+            if method == Method.POST:
+                resp = self._session.post(url, body, headers=headers)
+            elif method == Method.GET:
+                resp = self._session.get(url, headers=headers)
+            else:
+                raise ValueError("Request method should be POST or GET")
+        except ValueError as e:
+            raise e
+        except ConnectionError:
+            raise Exception("Request aborted. Instagram has probably blocked your fingerprint.")
+        except Exception as e:  # todo: narrow down
+            logger.exception(f"Exception while sending request to {url} with data: \n {body}")
+            raise e
+
+        self._check_response_for_errors(resp)
+        for func in self._request_finished_callbacks:
+            func(resp.headers)
+        return resp
+
+    def _build_body(self, data: Optional[Union[dict, list, bytes]], sign_request):
+        if sign_request:
+            as_json = self._json_dumps(data)
+            data = {
+                'ig_sig_key_version': self.ig_profile.signature_key_version,
+                'signed_body': f'SIGNATURE.{as_json}'
+            }
+        return data
+
+    def _check_response_for_errors(self, resp: requests.Response) -> None:
+        if resp.ok:
+            return
+
+        try:
+            parsed = self._json_loads(resp.text)
+        except orjson.JSONDecodeError:
+            if resp.status_code == 404 and '/friendships/' in resp.url:
+                raise InvalidUserId(f"account id: {resp.url.split('/')[-2]} is not recognized "
+                                    f"by Instagram or you do not have a relation with this account.")
+
+            logger.exception(f"response received: \n{resp.text}\nurl: {resp.url}\nstatus code: {resp.status_code}")
+            raise BadResponse("Received a non-200 response from Instagram")
+
+        message = parsed.get('message')
+        error_type = parsed.get('error_type')
+        two_factor_required = parsed.get('two_factor_required', False)
+
+        if two_factor_required:
+            return self._handle_2fa(parsed)
+        if error_type == 'bad_password':
+            raise IncorrectLoginDetails("Instagram does not recognize the provided login details")
+        if message in ("checkpoint_required", "challenge_required"):
+            if self._handle_checkpoint(resp):
+                return
+        if message == 'feedback_required':
+            self._handle_feedback_required(parsed)
+        if message == 'rate_limit_error':
+            raise TimeoutError("Calm down. Please try again in a few minutes.")
+        if message == 'Not authorized to view user':
+            raise AuthorizationError("This is a private user, which you do not follow.")
+        raise BadResponse("Received a non-200 response from Instagram")
+
+    def _handle_checkpoint(self, resp: Response) -> bool:
+        if not hasattr(self, '_handle_challenge'):
+            raise BadResponse("Challenge required. ChallengeMixin is not mixed in.")
+        return self._handle_challenge(resp)
+
+    def _handle_feedback_required(self, parsed: dict) -> None:
+        if os.environ.get("ENABLE_INSTAUTO_USAGE_METRICS", True):
+            # This logs which actions cause limitations on Instagram accounts.
+            # I use this data to focus my development on area's where it's most needed.
+            requests.post('https://instauto.rooy.dev/feedback_required', data={
+                'feedback_url': parsed.get('feedback_url'),
+                'category': parsed.get('category')
+            })
+        raise BadResponse("Something unexpected happened. Please check the IG app.")
+
+    def _request_preflight_check(self,
+                                 method: Method,
+                                 endpoint: str,
+                                 data: Optional[Union[dict, bytes]],
+                                 sign_request: bool
+                                 ) -> None:
+        if method == Method.GET:
+            assert not data
+        assert not endpoint.startswith('/')
+        if sign_request:
+            assert not isinstance(data, bytes)
+
+    def _prepare_url(self, endpoint: str, query: dict) -> str:
+        url = API_BASE_URL.format(endpoint) if 'https://' not in endpoint else endpoint
+        if query:
+            url += f"?{urllib.parse.urlencode(query)}"
+        return url
+
+    def _build_headers(self, add_default_headers: bool, headers, method):
+        if add_default_headers:
+            h = self._build_default_headers()
+            h.update(headers)
+            headers = h
+        if method == Method.POST:
+            headers['Content-Type'] = 'application/x-www-form-urlencoded; charset=UTF-8'
+        return headers
 
     def _build_user_agent(self) -> str:
         """Builds a user agent, making use from all required values in `self.ig_profile`, `self.device_profile` and
@@ -121,144 +244,5 @@ class RequestMixin:
         if public_api_key_id is not None: self.state.public_api_key_id = public_api_key_id
 
         public_api_key = headers.get('ig-set-password-encryption-pub-key')
-        if public_api_key is not None: self.state.public_api_key = public_api_key; self._encrypt_password()
+        if public_api_key is not None: self.state.public_api_key = public_api_key; self._encode_password()
 
-    def _request(self, endpoint: str, method: Method, query: dict = None, data: Union[dict, bytes] = None, headers: Dict[str, str]
-    = None, default_headers: bool = None, signed: bool = None) -> requests.Response:
-        """Creates and sends a request to the specified endpoint.
-
-        Parameters
-        ----------
-        endpoint : str
-            The endpoint that the request should be send to. `endpoint` should start with a non '/' character.
-            `endpoint` should end with a '/'.
-        method : {Method.Post, Method.Get}
-            Specifies which method to use for sending the HTTP request.
-        query : dict, optional
-            A dictionary that contains all key-value pairs that should be added to the final url, as a query string.
-        data : dict, optional
-            A dictionary that contains all key-value pairs that should be send along with a post request.
-        headers : dict, optional
-            A dictionary that contains all key-value pairs of the headers that should be sent along with the HTTP
-            request. Header values from this argument take priority over the default headers. Default headers are
-            overwritten if header values co-exist in both dict's.
-
-        Returns
-        -------
-        resp : requests.Response
-            The response that is returned by Instagram API.
-
-        Other Parameters
-        -------
-        default_headers : bool, optional
-            Defaults to True. If set to True, the default headers, `from ApiClient._build_default_headers`,
-            will be included in the request. This argument is pretty much only used in the initial request,
-            where we don't want to send headers, which we shouldn't know about at that point.
-
-        Raises
-        -------
-        WrongMethodException
-            When the `data` argument is provided, but the `method` argument is set to `Method.GET`. POST data cannot
-            be send along with GET requests. It is most likely that it either was mistaken for the `query` argument,
-            or that the method should be set to POST.
-        """
-        if query is None: query = {}
-        if data is None: data = {}
-        if default_headers is None: default_headers = True
-        if headers is None: headers = {}
-        if signed is None: signed = False
-
-        if endpoint.startswith('/'):
-            logger.warning("Are you sure that the endpoint starts with a slash?")
-
-        # This isn't the cleanest, but it works. This makes sure we can just pass in the full url for endpoints that
-        # do not start with /api/v1 (pretty much only for uploading pictures/videos), without adding an extra boolean
-        # or method to the function/class.
-        if 'https://' not in endpoint:
-            url = API_BASE_URL.format(endpoint)
-        else:
-            url = endpoint
-
-        if query:
-            url += f"?{urllib.parse.urlencode(query)}"
-        if default_headers:
-            h = self._build_default_headers()
-            h.update(headers)
-            headers = h
-
-        if method == Method.GET and data:
-            raise WrongMethodException("Conflicting information. ApiClient._request was called with the method set to"
-                                       "Method.GET, but was also provided the `data` argument. Data can only be used "
-                                       "with GET request, did you want to use Method.POST instead?")
-
-        if method == Method.POST:
-            headers['Content-Type'] = 'application/x-www-form-urlencoded; charset=UTF-8'
-
-        # pretty sure everything will works just fine if this is removed, but I haven't tasted it yet. TODO:
-        if signed:
-            as_json = json.dumps(data)
-            data = {
-                'ig_sig_key_version': self.ig_profile.signature_key_version,
-                'signed_body': f'SIGNATURE.{as_json}'
-            }
-
-        try:
-            if method == Method.POST:
-                resp = self._session.post(url, data, headers=headers)
-            elif method == Method.GET:
-                resp = self._session.get(url, headers=headers)
-            else:
-                raise ValueError("Request method should be POST or GET")
-
-            logger.debug(f"Sent request to {url}, method: {resp.request.method} with data: \n {data}")
-        except ValueError as e:
-            raise e
-        except Exception as e:  # todo: narrow down
-            logger.exception(f"Exception while sending request to {url} with data: \n {data}")
-            raise e
-
-        logger.debug(
-            f'{"*" * 20} START REQUEST {"*" * 20}\n'
-            f'METHOD: {resp.request.method}\n'
-            f'URL: {url}\n'
-            f'DATA: {data}\n'
-            f'HEADERS: {headers}\n'
-            f'RESPONSE: {resp.content}\n'
-            f'{"*" * 20} END REQUEST {"*" * 20}'
-        )
-
-        self._check_response_for_errors(resp)
-
-        for func in self._request_finished_callbacks:
-            func(resp.headers)
-
-        return resp
-
-    def _check_response_for_errors(self, resp: requests.Response) -> None:
-        try:
-            parsed = resp.json()
-        except json.JSONDecodeError:
-            if not resp.ok:
-                if resp.status_code == 404 and '/friendships/' in resp.url:
-                    raise InvalidUserId(f"account id: {resp.url.split('/')[-2]} is not recognized by Instagram or you do not have a relation with this account.")
-
-                logger.exception(f"response received: \n{resp.text}\nurl: {resp.url}\nstatus code: {resp.status_code}")
-                raise BadResponse("Received a non-200 response from Instagram")
-            return
-        if resp.ok:
-            return
-
-        if parsed.get('error_type') == 'bad_password':
-            raise IncorrectLoginDetails("Instagram does not recognize the provided login details")
-        if parsed.get('message') in ("checkpoint_required", "challenge_required"):
-            if not hasattr(self, '_handle_challenge'):
-                raise BadResponse("Challenge required. ChallengeMixin is not mixed in.")
-            eh = self._handle_challenge(resp)
-            if eh:
-                return
-        if parsed.get('message') == 'feedback_required':
-            # TODO: implement a handler for this error
-            raise BadResponse("Something unexpected happened. Please check the IG app.")
-        if parsed.get('message') == 'rate_limit_error':
-            raise TimeoutError("Calm down. Please try again in a few minutes.")
-        raise BadResponse("Received a non-200 response from Instagram")
