@@ -10,7 +10,7 @@ from pathlib import Path
 from dataclasses import dataclass, field
 from typing import Optional, List, Union
 
-from ..helpers import get_image_type
+from ..helpers import HelperMixin
 from instauto.api.structs import PostLocation
 from instauto.api.constants import DEFAULT_DEVICE_PROFILE
 
@@ -21,6 +21,52 @@ logger = logging.getLogger(__name__)
 # DATACLASSES
 #####################################
 
+@dataclass
+class UserTag:
+    """
+    Contains all information about a UserTag. This can be used to set the user tag for an UserTags object.
+
+    :param user_id: Id of the tagged user
+    :param x: relative x-coordinate with 0 <= x <= 1, with 0 for left and 1 for right
+    :param y: relative y-coordinate with 0 <= y <= 1, with 0 for top and 1 for bottom
+    """
+    user_id: str = ""
+    x: float = None
+    y: float = None
+
+    def to_dict(self) -> dict:
+        data = {
+            "user_id": self.user_id,
+            "position": [
+                round(self.x, ndigits=8),
+                round(self.y, ndigits=8)
+            ]
+        }
+        return data
+
+    def __repr__(self):
+        return pprint.pformat(self.to_dict())
+
+
+@dataclass
+class UserTags:
+    """
+    Contains all information about UserTags. This can be used to set usertags for an Instagram post.
+    :param usertags: this takes a list of UserTag objects
+    """
+    usertags: List[UserTag] = None
+
+    def to_dict(self) -> dict:
+        data = {
+            "in": []
+        }
+        for usertag in self.usertags:
+            data["in"].append(usertag)
+        return data
+
+    def __repr__(self):
+        return pprint.pformat(self.to_dict())
+
 
 @dataclass
 class Location:
@@ -117,7 +163,7 @@ class Save(_Base):
 
 
 class Comment(_Base):
-    action = 'save'
+    action = 'comment'
 
     def __init__(self, media_id: str, comment_text: str, container_module: str = "something", *args, **kwargs):
         self.comment_text = comment_text
@@ -125,6 +171,20 @@ class Comment(_Base):
         super().__init__(media_id=media_id, container_module=container_module, *args, **kwargs)
 
 
+class Archive(_Base):
+    action = 'only_me'
+
+    def __init__(self, media_id: str, container_module: str = "something", *args, **kwargs):
+        super().__init__(media_id=media_id, container_module=container_module, *args, **kwargs)
+
+
+class Unarchive(_Base):
+    action = 'undo_only_me'
+
+    def __init__(self, media_id: str, container_module: str = "something", *args, **kwargs):
+        super().__init__(media_id=media_id, container_module=container_module, *args, **kwargs)
+
+
 class UpdateCaption(_Base):
     action = 'edit_media'
 
@@ -136,10 +196,11 @@ class UpdateCaption(_Base):
 
 
 class _PostBase(cmmn.Base):
+    timezone_offset: str = ''
+
     def __init__(self, path: Union[str, Path], source_type: PostLocation, edits: Optional[Edits],
                  extra: Optional[Extra], device: Optional[Device], *args, **kwargs):
-        self.upload_id = str(time.time()).split('.')[0]
-        self.timezone_offset = str(time.localtime().tm_gmtoff)
+        self.upload_id = str(time.time() + random.randint(1, 100000)).split('.')[0]
         self.scene_capture_type = ''
         self.media_folder = 'Pictures'
         self.x_fb_waterfall_id = str(uuid.uuid4())
@@ -147,7 +208,7 @@ class _PostBase(cmmn.Base):
 
         self.source_type = source_type.value
 
-        image_type = get_image_type(path)
+        image_type = HelperMixin.get_image_type(path)
         # See issue #65
         if image_type not in ['jpg', 'jpeg']:
             raise ValueError("Instagram only accepts jpg/jpeg images")
@@ -185,14 +246,16 @@ class PostFeed(_PostBase):
     device_id: str = None
 
     def __init__(self, path: Union[str, Path], caption: str,
-                 location: Optional[Location] = None, edits: Optional[Edits] = None,
-                 extra: Optional[Extra] = None, device: Optional[Device] = None, *args, **kwargs):
+                 location: Optional[Location] = None, usertags: Optional[UserTags] = None,
+                 edits: Optional[Edits] = None, extra: Optional[Extra] = None,
+                 device: Optional[Device] = None, source_type: PostLocation = PostLocation.Feed, *args, **kwargs):
         self.suggested_venue_position = -1
         self.multi_sharing = '-1'
         self.caption = caption
         self.location = location
         self.size = imagesize.get(path)
-        super().__init__(path, PostLocation.Feed, edits, extra, device, *args, **kwargs)
+        self.user_tags = usertags
+        super().__init__(path, source_type, edits, extra, device, *args, **kwargs)
 
 
 class PostStory(_PostBase):
@@ -216,6 +279,18 @@ class PostStory(_PostBase):
         self._datapoint_from_client['device_id'] = lambda client: client.state.android_id
 
 
+class PostNull(PostFeed):
+    def __init__(self, path: Union[str, Path], edits: Optional[Edits] = None,
+                 extra: Optional[Extra] = None, device: Optional[Device] = None, *args, **kwargs):
+        super().__init__(path, None, None, None, edits, extra, device, PostLocation.Null, *args,**kwargs)
+
+
+class RetrieveById(cmmn.Base):
+    def __init__(self, media_id: str, *args, **kwargs) -> None:
+        self.media_id = media_id
+        super().__init__(*args, **kwargs)
+
+
 class RetrieveByUser(cmmn.Base):
     def __init__(self, user_id: str, exclude_comment: str = 'true', only_fetch_first_carousel_media: str = 'false',
                  *args, **kwargs):
@@ -234,16 +309,29 @@ class RetrieveByTag(cmmn.Base):
         self.only_fetch_first_carousel_media = only_fetch_first_carousel_media
         self.page = 0
         self.tag_name = tag_name
-
-
-class RetrieveLikers(_Base):
-    REQUEST = "post/get_likers.json"
-    def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
 
 
-class RetrieveCommenters(_Base):
-    REQUEST = "post/get_commenters.json"
+class RetrieveLikers(cmmn.Base):
+    REQUEST = "post/retrieve_likers.json"
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, media_id: str, *args, **kwargs):
+        self.media_id = media_id
         super().__init__(*args, **kwargs)
+
+
+class RetrieveCommenters(cmmn.Base):
+    REQUEST = "post/retrieve_commenters.json"
+
+    def __init__(self, media_id: str, *args, **kwargs):
+        self.media_id = media_id
+        super().__init__(*args, **kwargs)
+
+
+class RetrieveComments(cmmn.Base):
+    REQUEST = "post/retrieve_commenters.json"
+
+    def __init__(self, media_id: str, *args, **kwargs):
+        self.media_id = media_id
+        super().__init__(*args, **kwargs)
+
