@@ -9,15 +9,15 @@ logger = logging.getLogger(__name__)
 
 
 class SetGender(cmmn.Base):
-    _csrftoken: str = None
-    _uuid: str = None
+    _csrftoken: str = ''
+    _uuid: str = ''
 
     def __init__(self, gender: Optional[WhichGender] = None, custom_gender: Optional[str] = None, *args, **kwargs):
         if gender is None and custom_gender is None:
             raise ValueError("Either gender or custom_gender needs to be provided")
 
         self.gender = gender
-        self.custom_gender = custom_gender
+        self.custom_gender = custom_gender or ""
         super().__init__(*args, **kwargs)
 
 
@@ -35,12 +35,10 @@ class Update(cmmn.Base):
     _csrftoken: str = None
     _uid: str = None  # user id
     _uuid: str = None
-    raw_text: str = None
+    _csrftoken: str = None
 
     def __init__(self, external_url: Optional[str], phone_number: Optional[str] = None, username: Optional[str] = None,
                  first_name: Optional[str] = None, email: Optional[str] = None, *args, **kwargs):
-        if all([not x for x in [external_url, phone_number, username, first_name, email]]):
-            raise ValueError("No value(s) to update provided.")
         self.external_url = external_url
         self.phone_number = phone_number
         self.username = username
@@ -63,3 +61,13 @@ class Info(cmmn.Base):
             return f'users/{self.user_id}/info/'
         elif self.username:
             return f'users/{self.username}/usernameinfo/'
+
+
+class SetPicture(cmmn.Base):
+    _csrftoken: str = None
+    _uuid: str = None
+
+    def __init__(self, upload_id: int, *args, **kwargs):
+        self.upload_id = upload_id
+        self.use_fbuploader = True
+        super().__init__(*args, **kwargs)
