@@ -5,10 +5,10 @@ from time import sleep
 from typing import List, Dict, Tuple
 
 from instauto.api.client import ApiClient
-from instauto.api.exceptions import AuthorizationError
 from instauto.bot.input import Input
+from instauto.helpers import models
 from instauto.helpers.friendships import follow_user
-from instauto.helpers.post import like_post, comment_post, retrieve_posts_from_user
+from instauto.helpers.post import like_post, comment_post
 
 logger = logging.getLogger(__name__)
 
@@ -17,28 +17,38 @@ class Bot:
     stop: bool = False
     input: Input
     _actions: List = []
-    _post_cache: Dict[str, List[Dict]] = {}
 
-    def __init__(self, user_name: str, password: str, delay_between_action: float = 2.0) -> None:
+    def __init__(self, user_name: str, password: str, client: ApiClient = None,
+                 delay_between_action: float = 2.0, delay_variance: float = 0.0) -> None:
         """Initiate a new `Bot` instance.
 
         Args:
             user_name: the username of the account
             password: the password of the account
+            client: the `ApiClient` instance the Bot communicates with. If given, it will take precedence over credentials.
             delay_between_action: the amount of seconds to wait between actions (each like, follow, etc. is an action)
-        """
+            delay_variance: the amount of variance to add to the delay. Delay will be random number between (delay - variance) - (delay + variance).
+        """ 
+        
+        if client is not None:
+            self._client = client
+        else:
+            self._initialize_client_from_credentials(user_name, password)
+
+        self.input = Input(self._client)
+        self._actions = []
+        self._delay = delay_between_action if(delay_between_action) else 0
+        self._delay_variance = abs(delay_variance)
+        
+    def _initialize_client_from_credentials(self, user_name: str, password: str) -> None:
         instauto_save_path = f'.{user_name}.instauto.save'
         if os.path.isfile(instauto_save_path):
             self._client = ApiClient.initiate_from_file(instauto_save_path)
         else:
-            self._client = ApiClient(user_name=user_name, password=password)
-            self._client.login()
+            self._client = ApiClient(username=user_name, password=password)
+            self._client.log_in()
             self._client.save_to_disk(instauto_save_path)
 
-        self.input = Input(self._client)
-        self._actions = []
-        self._delay = delay_between_action
-
     def like(self, chance: int, amount: int) -> "Bot":
         """Like posts of users retrieved with the Input pipeline.
 
@@ -99,7 +109,7 @@ class Bot:
         or if the `stop` attribute is set to `True`."""
         accounts = self.input.filtered_accounts
         while not self.stop:
-            sleep(self._delay)
+            self._sleep_between_actions()
             account = accounts.pop(random.randint(0, len(accounts) - 1))
             for action in self._actions:
                 if random.randint(0, 100) > action['chance']:
@@ -123,17 +133,18 @@ class Bot:
                         action['func'](self._client, *args)
                     except Exception as e:
                         logger.warning("Caught exception: ", e)
+                  
+    def _sleep_between_actions(self):
+        min = (self._delay - self._delay_variance) if(self._delay - self._delay_variance) else 0
+        max = self._delay + self._delay_variance
+        sleeptime = round(random.uniform(min, max), 2)
+        sleep(sleeptime)
 
-    def _get_posts(self, account_name: str, force: bool = False) -> List[Dict]:
-        if account_name not in self._post_cache or force:
-            try:
-                self._post_cache[account_name] = retrieve_posts_from_user(self._client, 30, account_name)
-            except AuthorizationError:
-                logger.info(f"Can't retrieve posts from {account_name}. This is a private account.")
-        return self._post_cache.get(account_name)
+    def _get_posts(self, account_name: str, force: bool = False) -> List[models.Post]:
+        return self.input.get_posts(account_name, force)
 
     @staticmethod
-    def _resolve_args(args: Tuple, post: Dict = None, account: Dict = None) -> List:
+    def _resolve_args(args: Tuple, post: models.Post = None, account: models.User = None) -> List:
         a = list()
         for arg in args:
             if isinstance(arg, tuple) and callable(arg[0]):
@@ -142,7 +153,11 @@ class Bot:
                 a.append(arg)
         for i, arg in enumerate(a.copy()):
             if arg == 'POST_ID' and post is not None:
-                a[i] = post['pk']
+                a[i] = post.pk
             elif arg == 'ACCOUNT_ID' and account is not None:
-                a[i] = account['pk']
+                a[i] = account.pk
         return a
+
+    @classmethod
+    def from_client(cls, client: ApiClient, delay_between_action: float = 2.0, delay_variance: float = 0.0) -> "Bot":
+        return cls("", "", client=client, delay_between_action=delay_between_action, delay_variance=delay_variance)
