@@ -1,5 +1,6 @@
-from typing import Callable, List, Dict, Optional
+from typing import Callable, List, Dict
 from instauto.api.client import ApiClient
+from instauto.helpers import models
 from instauto.helpers.friendships import get_followers, get_following
 from instauto.helpers.search import get_user_id_from_username
 from instauto.helpers.post import get_likers_of_post, get_commenters_of_post
@@ -9,8 +10,8 @@ from instauto.helpers.post import retrieve_posts_from_user
 class Input:
     _input: List[Callable]
     _client: ApiClient
-    _post_cache: Dict[str, List[Dict]] = {}
-    _accounts: List[dict] = []
+    _post_cache: Dict[str, List[models.Post]] = {}
+    _accounts: List[models.User] = []
 
     def __init__(self, client: ApiClient):
         self._client = client
@@ -22,7 +23,10 @@ class Input:
             account_name: the account to retrieve from
             limit: the amount of accounts to retrieve
         """
-        following = get_following(self._client, get_user_id_from_username(self._client, account_name), limit)
+        user_id = get_user_id_from_username(self._client, account_name)
+        if user_id is None:
+            return self
+        following = get_following(self._client, str(user_id), limit)
         self._accounts.extend(following)
         return self
 
@@ -33,7 +37,10 @@ class Input:
             account_name: the account to retrieve from
             limit: the amount of accounts to retrieve
         """
-        followers = get_followers(self._client, get_user_id_from_username(self._client, account_name), limit)
+        user_id = get_user_id_from_username(self._client, account_name)
+        if user_id is None:
+            return self
+        followers = get_followers(self._client, str(user_id), limit)
         self._accounts.extend(followers)
         return self
 
@@ -45,7 +52,7 @@ class Input:
             limit: the amount of accounts to retrieve
         """
         likers = []
-        posts = self._get_posts(account_name)
+        posts = self.get_posts(account_name)
         for post in posts:
             likers.extend(get_likers_of_post(self._client, post['id']))
             if len(likers) > limit:
@@ -62,20 +69,29 @@ class Input:
             limit: the amount of accounts to retrieve
         """
         commenters = []
-        posts = self._get_posts(account_name)
+        posts = self.get_posts(account_name)
         for post in posts:
-            commenters.extend(get_commenters_of_post(self._client, post['id']))
+            commenters.extend(get_commenters_of_post(self._client, post.id))
             if len(commenters) > limit:
                 break
         self._accounts.extend(commenters[:limit:])
         return self
 
+    def from_user_list(self, accounts: List[dict]) -> "Input":
+        """Add supplied accounts to input
+        
+        Args:
+            accounts: List of account objects (objects/user.json)
+        """
+        self._accounts.extend(accounts)
+        return self
+
     @property
-    def filtered_accounts(self) -> List[dict]:
+    def filtered_accounts(self) -> List[models.User]:
         seen = []
         return list(filter(lambda x: x['id'] not in seen and seen.append(x['id']) is None, self._accounts))
 
-    def _get_posts(self, account_name: str, force: bool = False) -> List[Dict]:
+    def get_posts(self, account_name: str, force: bool = False) -> List[models.Post]:
         if account_name not in self._post_cache or force:
             self._post_cache[account_name] = retrieve_posts_from_user(self._client, 30, account_name)
         return self._post_cache.get(account_name)
